type DexieYProvider = import('y-dexie').DexieYProvider;
type YDoc = import('yjs').Doc;
export declare function useDocument(doc: YDoc | null | undefined): DexieYProvider | null;
export {};
//# sourceMappingURL=useDocument.d.ts.map